@@ -723,7 +723,7 @@ ProbingView<REAL>::propagateDomains()
 
          auto rowvec = consMatrix.getRowCoefficients( candrow );
 
-         propagate_row(candrow,
+         propagate_row( num, candrow,
              rowvec.getValues(), rowvec.getIndices(), rowvec.getLength(),
              probing_activities[candrow], lhs[candrow], rhs[candrow],
              rflags[candrow], probing_lower_bounds, probing_upper_bounds,
@@ -764,6 +764,10 @@ ProbingView<REAL>::propagateDomains()
 
                    REAL mindomred = isint ? minintdomred : mincontdomred;
 
+                   // Boundchanges on unbounded variables [lb, inf) or (-inf, ub]
+                   // are ignored since their fixing will probably not lead to
+                   // additional fixings nor infeasibility
+                   // but will trigger a huge sequence of bound changes
                    if( probing_domain_flags[colid].test(
                            ColFlag::kLbUseless ) ||
                        ( finiteDomain && delta > 0 &&
@@ -802,6 +806,10 @@ ProbingView<REAL>::propagateDomains()
 
                    REAL mindomred = isint ? minintdomred : mincontdomred;
 
+                   // Boundchanges on unbounded variables [lb, inf) or (-inf, ub]
+                   // are ignored since their fixing will probably not lead to
+                   // additional fixings nor infeasibility
+                   // but will trigger a huge sequence of bound changes
                    if( probing_domain_flags[colid].test(
                            ColFlag::kUbUseless ) ||
                        ( finiteDomain && delta > 0 &&
