@@ -227,11 +227,6 @@ presolve_and_solve(
          else if( presolve.getSATSolverFactory() &&
                   problem.test_problem_type(ProblemFlag::kBinary) )
          {
-            if( !presolve.getPresolveOptions().verification_with_VeriPB )
-            {
-               fmt::print( "please activate VeriPB to provide row scaling for SAT Solvers\n" );
-               return ResultStatus::kError;
-            }
             solver = presolve.getSATSolverFactory()->newSolver(
                 presolve.getVerbosityLevel() );
          }
@@ -244,6 +239,10 @@ presolve_and_solve(
             return ResultStatus::kError;
          }
       }
+      else
+      {
+         store_dual = true;
+      }
 
       auto result = presolve.apply( problem, store_dual );
 
@@ -298,6 +297,7 @@ presolve_and_solve(
             num.setFeasTol( REAL{ presolve.getPresolveOptions().feastol } );
             num.setEpsilon( REAL{ presolve.getPresolveOptions().epsilon } );
             num.setHugeVal( REAL{ presolve.getPresolveOptions().hugeval } );
+            num.setUseAbsFeas( presolve.getPresolveOptions().useabsfeas );
             bool success = OpbWriter<REAL>::writeProb( opts.reduced_problem_file, problem,
                                         result.postsolve.origcol_mapping, presolve.getRowScalingFactors(), num );
             //TODO: change name
@@ -318,6 +318,7 @@ presolve_and_solve(
       if( !opts.postsolve_archive_file.empty() )
       {
 
+#ifdef PAPILO_SERIALIZATION_AVAILABLE
          Timer t( writetime );
          std::ofstream ofs( opts.postsolve_archive_file,
                             std::ios_base::binary );
@@ -327,6 +328,9 @@ presolve_and_solve(
          oa << result.postsolve;
          fmt::print( "postsolve archive written to {} in {:.3f} seconds\n\n",
                      opts.postsolve_archive_file, t.getTime() );
+#else
+         fmt::print( "writing to potsolve archive is not possible without Boost Serialization!\n");
+#endif
       }
 
       if( opts.command == Command::kPresolve || problem.getNCols() == 0 )
@@ -597,6 +601,7 @@ template <typename REAL>
 void
 postsolve( const OptionsInfo& opts )
 {
+#ifdef PAPILO_SERIALIZATION_AVAILABLE
    PostsolveStorage<REAL> ps;
    std::ifstream inArchiveFile( opts.postsolve_archive_file,
                                 std::ios_base::binary );
@@ -670,6 +675,7 @@ postsolve( const OptionsInfo& opts )
 
 
    }
+#endif
    }
 
 } // namespace papilo
