@@ -188,70 +188,14 @@ ParallelColDetection<REAL>::findParallelCols(
    const Vec<REAL>& lbs = domains.lower_bounds;
    const Vec<REAL>& ubs = domains.upper_bounds;
 
-   auto checkDomainsForHoles = [&]( int col1, int col2, const REAL& scale2 ) {
-      // test whether we need to check that the domain of the merged
-      // column has no holes
-      assert( cflags[col1].test( ColFlag::kIntegral ) );
-      assert( cflags[col2].test( ColFlag::kIntegral ) );
-      assert( !cflags[col1].test( ColFlag::kLbInf, ColFlag::kUbInf ) );
-      assert( !cflags[col2].test( ColFlag::kLbInf, ColFlag::kUbInf ) );
+   auto checkDomainForHoles = [&]( int col, const REAL& scale )
+   {
+      assert( cflags[col].test( ColFlag::kIntegral ) );
+      assert( !cflags[col].test( ColFlag::kLbInf, ColFlag::kUbInf ) );
+      assert( num.isIntegral(scale) );
 
-      // compute the domains of the merged column
-      REAL mergeval = lbs[col2];
-      REAL mergeub = ubs[col2];
-
-      if( scale2 < 0 )
-      {
-         mergeval += scale2 * ubs[col1];
-         mergeub += scale2 * lbs[col1];
-      }
-      else
-      {
-         mergeval += scale2 * lbs[col1];
-         mergeub += scale2 * ubs[col1];
-      }
-
-      // scan domain of new variable for holes:
-      // test every value in domain of column 1 if it allows to
-      // find a value in the domain of column 2 to constitute the
-      // value the all values in the domain of the merged column.
-      // If no such value is found the columns cannot be merged
-      bool foundHole = false;
-      while( num.isLE( mergeval, mergeub ) )
-      {
-         // initialize col1val with the lower bound of column 1
-         REAL col1val = lbs[col1];
-
-         // test if col1val + scale2 * col2val = mergeval implies a
-         // value for col2val that is within the domain of column 2.
-         // If that is the case we can stop and increase mergeval by
-         // 1, otherwise we found a hole If that check failed for
-         // the current col1val we increase it by 1 the upper bound
-         // of column 1 is reached
-         foundHole = true;
-         while( num.isLE( col1val, ubs[col1] ) )
-         {
-            REAL col2val = mergeval - col1val * scale2;
-
-            if( num.isIntegral( col2val ) && num.isGE( col2val, lbs[col2] ) &&
-                num.isLE( col2val, ubs[col2] ) )
-            {
-               foundHole = false;
-               break;
-            }
-
-            col1val += 1;
-         }
-
-         // if a hole was found we can stop
-         if( foundHole )
-            break;
-
-         // test next value in domain
-         mergeval += 1;
-      }
-
-      return foundHole;
+      // there are holes iff the range of the unit column does not cover the scale
+      return num.isLT(ubs[col] - lbs[col], abs(scale) - 1);
    };
 
    if( constMatrix.getColumnCoefficients( bucket[0] ).getLength() <= 1 )
@@ -264,15 +208,17 @@ ParallelColDetection<REAL>::findParallelCols(
       assert(!is_binary);
       int col1 = bucket[0];
       auto col1vec = constMatrix.getColumnCoefficients( col1 );
-      const int length = col1vec.getLength();
       const REAL* coefs1 = col1vec.getValues();
+      const int length = col1vec.getLength();
+
       for( int j = 1; j < bucketSize; ++j )
       {
          int col2 = bucket[j];
          assert( !cflags[col2].test( ColFlag::kIntegral ) );
-         const REAL* coefs2 =
-             constMatrix.getColumnCoefficients( col2 ).getValues();
-         if( check_parallelity( num, obj, col1, length, coefs1, col2, coefs2 ) )
+         const REAL* coefs2 = constMatrix.getColumnCoefficients( col2 ).getValues();
+         assert( num.isLE( abs( coefs1[0] ), abs( coefs2[0] ) ) );
+
+         if( check_parallelity( num, obj, col2, length, coefs2, col1, coefs1 ) )
          {
             TransactionGuard<REAL> tg{ reductions };
             reductions.lockCol( col2 );
@@ -291,39 +237,44 @@ ParallelColDetection<REAL>::findParallelCols(
       bool abort_after_first_loop = !is_binary;
       int first_col_with_finite_bounds = -1;
 
-      for( int i = 0; i < bucketSize; i++ )
+      for( int i = 0; i < bucketSize; ++i )
       {
          int col1 = bucket[i];
+         assert( cflags[col1].test( ColFlag::kIntegral ) );
+
          if( cflags[col1].test( ColFlag::kLbInf, ColFlag::kUbInf ) )
             continue;
+
          if( first_col_with_finite_bounds == -1 )
             first_col_with_finite_bounds = i;
+
          if( i == first_col_with_finite_bounds + 1 && abort_after_first_loop )
             break;
+
          auto col1vec = constMatrix.getColumnCoefficients( col1 );
-         const int length = col1vec.getLength();
          const REAL* coefs1 = col1vec.getValues();
-         assert( cflags[col1].test( ColFlag::kIntegral ) );
+         const int length = col1vec.getLength();
+
          for( int j = i + 1; j < bucketSize; ++j )
          {
             int col2 = bucket[j];
-            const REAL* coefs2 =
-                constMatrix.getColumnCoefficients( col2 ).getValues();
             assert( cflags[col2].test( ColFlag::kIntegral ) );
-            assert( num.isLE( abs( coefs1[0] ), abs( coefs2[0] ) ) );
-
 
             if( cflags[col2].test( ColFlag::kLbInf, ColFlag::kUbInf ) )
                continue;
 
-            bool parallel = check_parallelity( num, obj, col1, length, coefs1,
-                                               col2, coefs2 );
-            if( parallel &&
-//                !symmetries.contains_symmetry(col1, col2) &&
-                !checkDomainsForHoles( col1, col2, coefs1[0] / coefs2[0] ) )
+            auto col2vec = constMatrix.getColumnCoefficients( col2 );
+            assert( col2vec.getLength() == length );
+            const REAL* coefs2 = col2vec.getValues();
+            assert( num.isLE( abs( coefs1[0] ), abs( coefs2[0] ) ) );
+            REAL scale = coefs2[0] / coefs1[0];
+
+            if( num.isIntegral(scale)
+                  && check_parallelity( num, obj, col2, length, coefs2, col1, coefs1 )
+                  && !checkDomainForHoles( col1, scale )
+//                  && !symmetries.contains_symmetry( col2, col1 )
+                  )
             {
-               if( is_binary && abs( coefs1[0] / coefs2[0] ) != 1 )
-                  continue;
                TransactionGuard<REAL> tg{ reductions };
                reductions.lockCol( col2 );
                reductions.lockCol( col1 );
@@ -340,36 +291,35 @@ ParallelColDetection<REAL>::findParallelCols(
       assert(!is_binary);
       int col1 = bucket[0];
       auto col1vec = constMatrix.getColumnCoefficients( col1 );
+      const REAL* coefs1 = col1vec.getValues();
       const int length = col1vec.getLength();
-      int continuous_cols;
+      int ncontinuous_cols;
 
       assert( !cflags[col1].test( ColFlag::kIntegral ) );
       {
-         const REAL* coefs1 = col1vec.getValues();
-
          Vec<std::pair<int, int>> continuous_parallel_cols;
-         for( continuous_cols = 1; continuous_cols < bucketSize;
-              ++continuous_cols )
+
+         for( ncontinuous_cols = 1; ncontinuous_cols < bucketSize; ++ncontinuous_cols )
          {
-            int col2 = bucket[continuous_cols];
+            int col2 = bucket[ncontinuous_cols];
+
             if( cflags[col2].test( ColFlag::kIntegral ) )
                break;
-            const REAL* coefs2 =
-                constMatrix.getColumnCoefficients( col2 ).getValues();
-            if( check_parallelity( num, obj, col1, length, coefs1, col2,
-                                   coefs2 ) )
-            {
+
+            const REAL* coefs2 = constMatrix.getColumnCoefficients( col2 ).getValues();
+            assert( num.isLE( abs( coefs1[0] ), abs( coefs2[0] ) ) );
+
+            if( check_parallelity( num, obj, col2, length, coefs2, col1, coefs1 ) )
                continuous_parallel_cols.emplace_back( col2, col1 );
-            }
          }
+
          // TODO: use the updated bounds of the cont parallel columns
-         int col2 = bucket[continuous_cols];
+         int col2 = bucket[ncontinuous_cols];
          assert( cflags[col2].test( ColFlag::kIntegral ) );
-         const REAL* coefs2 =
-             constMatrix.getColumnCoefficients( col2 ).getValues();
-         if( ( can_be_merged( num, lbs, ubs, col1, coefs1, coefs2,
-                              cflags ) ) &&
-             check_parallelity( num, obj, col1, length, coefs1, col2, coefs2 ) )
+         const REAL* coefs2 = constMatrix.getColumnCoefficients( col2 ).getValues();
+
+         if( can_be_merged( num, lbs, ubs, col1, coefs1, coefs2, cflags )
+               && check_parallelity( num, obj, col1, length, coefs1, col2, coefs2 ) )
          {
             TransactionGuard<REAL> tg{ reductions };
             reductions.lockCol( col1 );
@@ -392,32 +342,43 @@ ParallelColDetection<REAL>::findParallelCols(
             }
          }
       }
-      for( int int_cols = continuous_cols; int_cols < bucketSize; ++int_cols )
+
+      for( int i = ncontinuous_cols; i < bucketSize; ++i )
       {
-         int int_col = bucket[int_cols];
-         auto int_col1vec = constMatrix.getColumnCoefficients( int_col );
-         assert( cflags[int_col].test( ColFlag::kIntegral ) );
-         assert( int_col1vec.getLength() == length );
-         const REAL* int_coefs1 = int_col1vec.getValues();
-         for( int j = int_cols + 1; j < bucketSize; ++j )
+         col1 = bucket[i];
+         assert( cflags[col1].test( ColFlag::kIntegral ) );
+
+         if( cflags[col1].test( ColFlag::kLbInf, ColFlag::kUbInf ) )
+            continue;
+
+         col1vec = constMatrix.getColumnCoefficients( col1 );
+         assert( col1vec.getLength() == length );
+         coefs1 = col1vec.getValues();
+
+         for( int j = i + 1; j < bucketSize; ++j )
          {
             int col2 = bucket[j];
-            const REAL* coefs2 =
-                constMatrix.getColumnCoefficients( col2 ).getValues();
+            assert( cflags[col2].test( ColFlag::kIntegral ) );
 
-            if( cflags[int_col].test( ColFlag::kLbInf, ColFlag::kUbInf ) ||
-                cflags[col2].test( ColFlag::kLbInf, ColFlag::kUbInf ) )
+            if( cflags[col2].test( ColFlag::kLbInf, ColFlag::kUbInf ) )
                continue;
 
-            bool parallel = check_parallelity( num, obj, int_col, length,
-                                               int_coefs1, col2, coefs2 );
-            if( parallel && !checkDomainsForHoles(
-                                 int_col, col2, int_coefs1[0] / coefs2[0] ) )
+            auto col2vec = constMatrix.getColumnCoefficients( col2 );
+            assert( col2vec.getLength() == length );
+            const REAL* coefs2 = col2vec.getValues();
+            assert( num.isLE( abs( coefs1[0] ), abs( coefs2[0] ) ) );
+            REAL scale = coefs2[0] / coefs1[0];
+
+            if( num.isIntegral(scale)
+                  && check_parallelity( num, obj, col2, length, coefs2, col1, coefs1 )
+                  && !checkDomainForHoles( col1, scale )
+//                  && !symmetries.contains_symmetry( col2, col1 )
+                  )
             {
                TransactionGuard<REAL> tg{ reductions };
                reductions.lockCol( col2 );
-               reductions.lockCol( int_col );
-               reductions.mark_parallel_cols( col2, int_col );
+               reductions.lockCol( col1 );
+               reductions.mark_parallel_cols( col2, col1 );
             }
          }
       }
@@ -576,6 +537,15 @@ ParallelColDetection<REAL>::execute( const Problem<REAL>& problem,
        col.get(), col.get() + ncols,
        [&]( int a, int b )
        {
+          if( cflags[a].test( ColFlag::kInactive ) && cflags[b].test( ColFlag::kInactive ) )
+             return a < b;
+          if( cflags[a].test( ColFlag::kInactive ) )
+             return true;
+          if( cflags[b].test( ColFlag::kInactive ) )
+             return false;
+          assert(constMatrix.getColumnCoefficients( a ).getLength() > 0);
+          assert(constMatrix.getColumnCoefficients( b ).getLength() > 0);
+
           if( supportid[a] < supportid[b] ||
               ( supportid[a] == supportid[b] && coefhash[a] < coefhash[b] ) )
              return true;
@@ -588,6 +558,7 @@ ParallelColDetection<REAL>::execute( const Problem<REAL>& problem,
           bool flag_b_integer = cflags[b].test( ColFlag::kIntegral );
           if( flag_a_integer != flag_b_integer )
              return !flag_a_integer;
+
           return // sort by scale factor
               abs( obj[a] ) < abs( obj[b] ) ||
               // sort by scale factor if obj is zero
