@@ -3,8 +3,7 @@
 /*               This file is part of the program and library                */
 /*    PaPILO --- Parallel Presolve for Integer and Linear Optimization       */
 /*                                                                           */
-/* Copyright (C) 2020-2022 Konrad-Zuse-Zentrum                               */
-/*                     fuer Informationstechnik Berlin                       */
+/* Copyright (C) 2020-2024 Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
 /* This program is free software: you can redistribute it and/or modify      */
 /* it under the terms of the GNU Lesser General Public License as published  */
@@ -42,12 +41,29 @@ class SingletonCols : public PresolveMethod<REAL>
    {
       this->setName( "colsingleton" );
       this->setTiming( PresolverTiming::kFast );
+      this->setArgument( ArgumentType::kAggregation );
    }
 
    virtual PresolveStatus
    execute( const Problem<REAL>& problem,
-            const ProblemUpdate<REAL>& problemUpdate, const Num<REAL>& num,
-            Reductions<REAL>& reductions, const Timer& timer) override;
+            const ProblemUpdate<REAL>& problemUpdate,
+            const Num<REAL>& num, Reductions<REAL>& reductions,
+            const Timer& timer, int& reason_of_infeasibility) override;
+
+ private:
+   /**
+       * if substituting a non-implied singleton variable scaling should be applied to the constraint to
+       * avoid numerical difficulties since remaining constraint can be violated by a small margin translating to greater error
+       * in the variable bound.
+       * This scaling affects dual-postsolve in which it is currently not considered and therefore then disabled.
+    */
+   REAL
+   scale_and_shift( bool is_primal, REAL side, REAL var_bound, REAL val )
+   {
+      if( is_primal )
+         return side / abs( val ) - ( val > 0 ? var_bound : -var_bound );
+      return side - var_bound * val;
+   };
 };
 
 #ifdef PAPILO_USE_EXTERN_TEMPLATES
@@ -60,9 +76,8 @@ template <typename REAL>
 PresolveStatus
 SingletonCols<REAL>::execute( const Problem<REAL>& problem,
                               const ProblemUpdate<REAL>& problemUpdate,
-                              const Num<REAL>& num,
-                              Reductions<REAL>& reductions, const Timer& timer )
-{
+                              const Num<REAL>& num, Reductions<REAL>& reductions,
+                              const Timer& timer, int& reason_of_infeasibility){
    const auto& domains = problem.getVariableDomains();
    const auto& lower_bounds = domains.lower_bounds;
    const auto& upper_bounds = domains.upper_bounds;
@@ -80,7 +95,9 @@ SingletonCols<REAL>::execute( const Problem<REAL>& problem,
 
    PresolveStatus result = PresolveStatus::kUnchanged;
 
-   auto handleEquation = [&]( int col, bool lowerboundImplied, bool ubimplied,
+   bool is_primal = problemUpdate.getPostsolveType() == PostsolveType::kPrimal;
+
+   auto handleEquation = [&]( int col, bool lbimplied, bool ubimplied,
                               const REAL& val, int row, bool impliedeq,
                               const REAL& side ) {
       if( !impliedeq && rowSizes[row] <= 1 )
@@ -115,80 +132,73 @@ SingletonCols<REAL>::execute( const Problem<REAL>& problem,
       reductions.substituteColInObjective( col, row );
 
       // now check if the equation is redundant or needs to be modified
-      if( lowerboundImplied && ubimplied )
+      if( lbimplied && ubimplied )
       {
          // implied free -> just remove the equation completely
          reductions.markRowRedundant( row );
       }
       else
       {
-         assert( lowerboundImplied || !cflags[col].test( ColFlag::kLbInf ) );
+         REAL scaled_lhs = side;
+         REAL scaled_rhs = side;
+
+         assert( lbimplied || !cflags[col].test( ColFlag::kLbInf ) );
          assert( ubimplied || !cflags[col].test( ColFlag::kUbInf ) );
 
          // implied free only for one bound -> modify equation to be an
          // inequality and remove the columns coefficient
          reductions.changeMatrixEntry( row, col, 0 );
 
-         if( num.isLT(val, 0) )
+         if( val < 0 )
          {
-            if( num.isLE(upper_bounds[col], 0) && !cflags[col].test( ColFlag::kUbInf ) )
-            {
-
-               if( lowerboundImplied )
-                  reductions.changeRowLHSInf( row );
-               else if( !num.isZero(lower_bounds[col]) )
-                  reductions.changeRowLHS( row,
-                                           side - lower_bounds[col] * val );
-               if( ubimplied )
-                  reductions.changeRowRHSInf( row );
-               else if( !num.isZero(upper_bounds[col]) )
-                  reductions.changeRowRHS( row,
-                                           side - upper_bounds[col] * val );
-            }
+            if( lbimplied )
+               reductions.changeRowLHSInf( row );
             else
-            {
-               if( ubimplied )
-                  reductions.changeRowRHSInf( row );
-               else if( !num.isZero(upper_bounds[col]) )
-                  reductions.changeRowRHS( row,
-                                           side - upper_bounds[col] * val );
-               if( lowerboundImplied )
-                  reductions.changeRowLHSInf( row );
-               else if( !num.isZero(lower_bounds[col]) )
-                  reductions.changeRowLHS( row,
-                                           side - lower_bounds[col] * val );
-            }
+               scaled_lhs = scale_and_shift( is_primal, side, lower_bounds[col], val );
+
+            if( ubimplied )
+               reductions.changeRowRHSInf( row );
+            else
+               scaled_rhs = scale_and_shift( is_primal, side, upper_bounds[col], val );
          }
          else
          {
-            if( num.isLE(upper_bounds[col], 0) && !cflags[col].test( ColFlag::kUbInf ) )
-            {
-               if( lowerboundImplied )
-                  reductions.changeRowRHSInf( row );
-               else if( !num.isZero(lower_bounds[col]))
-                  reductions.changeRowRHS( row,
-                                           side - lower_bounds[col] * val );
-
-               if( ubimplied )
-                  reductions.changeRowLHSInf( row );
-               else if( !num.isZero(upper_bounds[col]) )
-                  reductions.changeRowLHS( row,
-                                           side - upper_bounds[col] * val );
-            }
+            if( ubimplied )
+               reductions.changeRowLHSInf( row );
             else
+               scaled_lhs = scale_and_shift( is_primal, side, upper_bounds[col], val );
+
+            if( lbimplied )
+               reductions.changeRowRHSInf( row );
+            else
+               scaled_rhs = scale_and_shift( is_primal, side, lower_bounds[col], val );
+         }
+
+         // avoid temporary infeasibility
+         if( scaled_lhs > side )
+         {
+            if( scaled_rhs != side )
+               reductions.changeRowRHS( row, scaled_rhs );
+
+            reductions.changeRowLHS( row, scaled_lhs );
+         }
+         else
+         {
+            if( scaled_lhs < side )
+               reductions.changeRowLHS( row, scaled_lhs );
+
+            if( scaled_rhs != side )
+               reductions.changeRowRHS( row, scaled_rhs );
+         }
+
+         if( is_primal && abs( val ) != 1 )
+         {
+            auto rowdata = constMatrix.getRowCoefficients( row );
+            for( int i = 0; i < rowdata.getLength(); i++ )
             {
-               if( ubimplied )
-                  reductions.changeRowLHSInf( row );
-               else if( !num.isZero(upper_bounds[col])  )
-                  reductions.changeRowLHS( row,
-                                           side - upper_bounds[col] * val );
-
-               if( lowerboundImplied )
-                  reductions.changeRowRHSInf( row );
-
-               else if( !num.isZero(lower_bounds[col]) )
-                  reductions.changeRowRHS( row,
-                                           side - lower_bounds[col] * val );
+               int col_index = rowdata.getIndices()[i];
+               if( col_index != col )
+                  reductions.changeMatrixEntry( row, col_index, rowdata.getValues()[i] / abs(val) );
             }
          }
       }
@@ -220,15 +230,15 @@ SingletonCols<REAL>::execute( const Problem<REAL>& problem,
          // Found singleton column within an equation:
          // Check if it is implied free on one bound. In that case the
          // variable is substituted and the constraint stays as an inequality
-         // constraint. Otherwise it is equaivalent to implied free variable
+         // constraint. Otherwise, it is equivalent to implied free variable
          // substitution.
 
-         bool lowerboundImplied =
+         bool lbimplied =
              row_implies_LB( num, lhs_values[row], rhs_values[row], rflags[row],
                              activities[row], val, lower_bounds[col],
                              upper_bounds[col], cflags[col] );
 
-         if( !lowerboundImplied &&
+         if( !lbimplied &&
              !problemUpdate.getPresolveOptions().removeslackvars )
             continue;
 
@@ -239,7 +249,7 @@ SingletonCols<REAL>::execute( const Problem<REAL>& problem,
 
          if( !ubimplied &&
              ( !problemUpdate.getPresolveOptions().removeslackvars ||
-               ( !lowerboundImplied && !num.isZero(obj[col]) ) ) )
+               ( !lbimplied && !num.isZero(obj[col]) ) ) )
             continue;
 
          if( cflags[col].test( ColFlag::kIntegral ) )
@@ -266,8 +276,7 @@ SingletonCols<REAL>::execute( const Problem<REAL>& problem,
                continue;
          }
 
-         handleEquation( col, lowerboundImplied, ubimplied, val, row, false,
-                         rhs_values[row] );
+         handleEquation( col, lbimplied, ubimplied, val, row, false, rhs_values[row] );
 
          continue;
       }
@@ -349,7 +358,7 @@ SingletonCols<REAL>::execute( const Problem<REAL>& problem,
          REAL duallb = obj[col] / val;
          REAL dualub = duallb;
 
-         bool lowerboundImplied =
+         bool lbimplied =
              row_implies_LB( num, lhs_values[row], rhs_values[row], rflags[row],
                              activities[row], val, lower_bounds[col],
                              upper_bounds[col], cflags[col] );
@@ -358,12 +367,12 @@ SingletonCols<REAL>::execute( const Problem<REAL>& problem,
                              activities[row], val, lower_bounds[col],
                              upper_bounds[col], cflags[col] );
 
-         if( lowerboundImplied && ubimplied )
+         if( lbimplied && ubimplied )
          {
             duallbinf = false;
             dualubinf = false;
          }
-         else if( lowerboundImplied )
+         else if( lbimplied )
          {
             if( num.isGT(val, 0) )
                duallbinf = false;
@@ -388,14 +397,14 @@ SingletonCols<REAL>::execute( const Problem<REAL>& problem,
 
             // check again if row implies more bounds with new right hand
             // side
-            if( !lowerboundImplied )
+            if( !lbimplied )
             {
-               lowerboundImplied = row_implies_LB(
+               lbimplied = row_implies_LB(
                    num, lhs_values[row], lhs_values[row], RowFlag::kEquation,
                    activities[row], val, lower_bounds[col], upper_bounds[col],
                    cflags[col] );
 
-               if( !lowerboundImplied &&
+               if( !lbimplied &&
                    !problemUpdate.getPresolveOptions().removeslackvars )
                   removevar = false;
             }
@@ -409,15 +418,12 @@ SingletonCols<REAL>::execute( const Problem<REAL>& problem,
 
                if( !ubimplied &&
                    ( !problemUpdate.getPresolveOptions().removeslackvars ||
-                     ( !lowerboundImplied && !num.isZero(obj[col]) ) ) )
+                     ( !lbimplied && !num.isZero(obj[col]) ) ) )
                   removevar = false;
             }
 
             if( removevar )
-            {
-               handleEquation( col, lowerboundImplied, ubimplied, val, row,
-                               true, lhs_values[row] );
-            }
+               handleEquation( col, lbimplied, ubimplied, val, row, true, lhs_values[row] );
             else
             {
                // if the variable should not be removed, then just apply the
@@ -439,14 +445,14 @@ SingletonCols<REAL>::execute( const Problem<REAL>& problem,
                     rhs_values[row] != lhs_values[row] );
 
             // check again if row implies more bounds with new left hand side
-            if( !lowerboundImplied )
+            if( !lbimplied )
             {
-               lowerboundImplied = row_implies_LB(
+               lbimplied = row_implies_LB(
                    num, rhs_values[row], rhs_values[row], RowFlag::kEquation,
                    activities[row], val, lower_bounds[col], upper_bounds[col],
                    cflags[col] );
 
-               if( !lowerboundImplied &&
+               if( !lbimplied &&
                    !problemUpdate.getPresolveOptions().removeslackvars )
                   removevar = false;
             }
@@ -460,15 +466,12 @@ SingletonCols<REAL>::execute( const Problem<REAL>& problem,
 
                if( !ubimplied &&
                    ( !problemUpdate.getPresolveOptions().removeslackvars ||
-                     ( !lowerboundImplied && !num.isZero(obj[col]) ) ) )
+                     ( !lbimplied && !num.isZero(obj[col]) ) ) )
                   removevar = false;
             }
 
             if( removevar )
-            {
-               handleEquation( col, lowerboundImplied, ubimplied, val, row,
-                               true, rhs_values[row] );
-            }
+               handleEquation( col, lbimplied, ubimplied, val, row, true, rhs_values[row] );
             else
             {
                // if the variable should not be removed, then just apply the
